﻿using System.Windows;
using System.Windows.Input;
using System.Windows.Media;

namespace NwWPFVisualOnWindow {
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window {
		private readonly VisualCollection _myVisualCollection;
		private readonly MyVisual _myVisual = new();
		private readonly MyVisualRotating _myVisualRotating = new();
		private readonly RotateTransform _rotation = new(0, 250, 110);

		public MainWindow() {
			InitializeComponent();
			Title = "Click this window and you can hide/show the rotating shape.";
			_myVisualCollection = new(this);
			_myVisualCollection.Add(_myVisual);
			_myVisualRotating.Transform = _rotation;
			_myVisualCollection.Add(_myVisualRotating);
			CompositionTarget.Rendering += CompositionTarget_Rendering;
		}
		protected override int VisualChildrenCount {
			get { return base.VisualChildrenCount + _myVisualCollection.Count; }
		}
		protected override Visual GetVisualChild(int index) {
			if(index < base.VisualChildrenCount)
				return base.GetVisualChild(0);
			else
				return _myVisualCollection[index - base.VisualChildrenCount];
		}
		protected override void OnMouseDown(MouseButtonEventArgs e) {
			base.OnMouseDown(e);
			if(_myVisualCollection.Contains(_myVisualRotating))
				_myVisualCollection.Remove(_myVisualRotating);
			else
				_myVisualCollection.Add(_myVisualRotating);
		}
		private void CompositionTarget_Rendering(object? sender, System.EventArgs e) {
			var angle = _rotation.Angle;
			angle += 2.0; if(angle > 360) angle -= 360;
			_rotation.Angle = angle;
		}
	}
	public class MyVisual : DrawingVisual {
		public MyVisual() {
			using var dc = RenderOpen();
			dc.DrawEllipse(Brushes.Blue, null, new(75, 50), 50, 30);
			dc.DrawLine(new Pen(Brushes.Red, 1), new(10, 100), new(100, 100));
		}
	}
	public class MyVisualRotating : DrawingVisual {
		public MyVisualRotating() {
			using var dc = RenderOpen();
			dc.DrawRoundedRectangle(null, new Pen(Brushes.Black, 5), new Rect(new Point(200, 10), new Size(100, 200)), 25, 25);
		}
	}
}